#
# Copyright (c) 2008-2019 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

from pyramid.decorator import reify
from pyramid.events import subscriber
from pyramid.view import view_config
from z3c.form import field
from z3c.form.interfaces import DISPLAY_MODE, IDataExtractedEvent
from z3c.table.column import GetAttrColumn
from z3c.table.interfaces import IColumn, IValues
from zope.interface import Invalid

from onf_website import _
from onf_website.shared.contact.interfaces.theme import IContactTheme, IContactThemeManager, \
    IContactThemeManagerTarget
from onf_website.shared.contact.theme import ContactTheme
from pyams_content.interfaces import MANAGE_TOOL_PERMISSION
from pyams_content.zmi import pyams_content
from pyams_form.form import AJAXAddForm, ajax_config
from pyams_form.security import ProtectedFormObjectMixin
from pyams_i18n.interfaces import II18n
from pyams_pagelet.pagelet import pagelet_config
from pyams_skin.container import ContainerView, delete_container_element
from pyams_skin.event import get_json_table_row_refresh_event
from pyams_skin.interfaces.viewlet import IWidgetTitleViewletManager
from pyams_skin.layer import IPyAMSLayer
from pyams_skin.table import ActionColumn, BaseTable, I18nColumn, TrashColumn
from pyams_skin.viewlet.menu import MenuItem
from pyams_skin.viewlet.toolbar import ToolbarAction
from pyams_utils.adapter import ContextRequestViewAdapter, adapter_config
from pyams_utils.fanstatic import get_resource_path
from pyams_utils.traversing import get_parent
from pyams_utils.unicode import translate_string
from pyams_utils.url import absolute_url
from pyams_viewlet.viewlet import viewlet_config
from pyams_zmi.form import AdminDialogAddForm, AdminDialogEditForm
from pyams_zmi.interfaces.menu import IPropertiesMenu
from pyams_zmi.layer import IAdminLayer
from pyams_zmi.view import InnerAdminView


#
# Contact themes manager view
#

@viewlet_config(name='contact-themes.menu', context=IContactThemeManagerTarget, layer=IAdminLayer,
                manager=IPropertiesMenu, permission=MANAGE_TOOL_PERMISSION, weight=6)
class ContactThemeManagerMenu(MenuItem):
    """Contact themes manager menu"""

    label = _("Contact themes")
    icon_class = 'fa-tags'
    url = '#contact-themes.html'


class ContactThemeManagerTable(ProtectedFormObjectMixin, BaseTable):
    """Contact themes manager table"""

    prefix = 'themes'
    title = _("Content themes")

    sortOn = None

    @property
    def cssClasses(self):
        classes = ['table', 'table-bordered', 'table-striped', 'table-hover',
                   'table-tight', 'datatable']
        return {'table': ' '.join(classes)}

    @property
    def data_attributes(self):
        attributes = super(ContactThemeManagerTable, self).data_attributes
        attributes['table'] = {
            'id': self.id,
            'data-ams-plugins': 'pyams_content',
            'data-ams-plugin-pyams_content-src': get_resource_path(pyams_content),
            'data-ams-location': absolute_url(IContactThemeManager(self.context), self.request)
        }
        return attributes

    @reify
    def values(self):
        return list(super(ContactThemeManagerTable, self).values)


@adapter_config(context=(IContactThemeManagerTarget, IPyAMSLayer, ContactThemeManagerTable),
                provides=IValues)
class ContactThemeManagerValues(ContextRequestViewAdapter):
    """Contact themes manager values adapter"""

    @property
    def values(self):
        return IContactThemeManager(self.context).values()


@adapter_config(name='name',
                context=(IContactThemeManagerTarget, IPyAMSLayer, ContactThemeManagerTable),
                provides=IColumn)
class ContactThemeManagerTableNameColumn(I18nColumn, GetAttrColumn):
    """Contact themes manager name column"""

    _header = _("Selection label (or name)")
    attrName = 'name'
    weight = 10

    dt_sort_type = 'string'

    def getValue(self, obj):
        i18n = II18n(obj)
        value = i18n.query_attribute('label', request=self.request)
        if not value:
            value = '<i>{}</i>'.format(
                super(ContactThemeManagerTableNameColumn, self).getValue(obj))
        return value


@adapter_config(name='associations',
                context=(IContactThemeManagerTarget, IPyAMSLayer, ContactThemeManagerTable),
                provides=IColumn)
class ContactThemeManagerTableAssociationsColumn(ActionColumn):
    """Contact themes manager associations column"""

    weight = 110

    icon_class = 'fa fa-fw fa-link'
    icon_hint = _("Theme associations")

    url = 'associations-dialog.html'
    modal_target = True

    permission = MANAGE_TOOL_PERMISSION


@adapter_config(name='trash',
                context=(IContactThemeManagerTarget, IPyAMSLayer, ContactThemeManagerTable),
                provides=IColumn)
class ContactThemeManagerTableTrashColumn(TrashColumn):
    """Contact themes manager table trash column"""

    permission = MANAGE_TOOL_PERMISSION


@view_config(name='delete-element.json', context=IContactThemeManager, request_type=IPyAMSLayer,
             permission=MANAGE_TOOL_PERMISSION, renderer='json', xhr=True)
def delete_data_type(request):
    """Contact theme delete view"""
    return delete_container_element(request, ignore_permission=True)


@pagelet_config(name='contact-themes.html', context=IContactThemeManagerTarget, layer=IPyAMSLayer,
                permission=MANAGE_TOOL_PERMISSION)
class ContactThemeManagerView(InnerAdminView, ContainerView):
    """Contact themes manager view"""

    table_class = ContactThemeManagerTable


#
# Contact themes views
#

@viewlet_config(name='add-contact-theme.action', context=IContactThemeManagerTarget,
                layer=IAdminLayer, view=ContactThemeManagerTable, manager=IWidgetTitleViewletManager,
                permission=MANAGE_TOOL_PERMISSION, weight=1)
class ContactThemeAddAction(ToolbarAction):
    """Contact theme add action"""

    label = _("Add contact theme")
    label_css_class = 'fa fa-fw fa-plus'

    url = 'add-contact-theme.html'
    modal_target = True


@pagelet_config(name='add-contact-theme.html', context=IContactThemeManagerTarget,
                layer=IPyAMSLayer, permission=MANAGE_TOOL_PERMISSION)
@ajax_config(name='add-contact-theme.json', context=IContactThemeManagerTarget,
             layer=IPyAMSLayer, base=AJAXAddForm)
class ContactThemeAddForm(AdminDialogAddForm):
    """Contact theme add form"""

    legend = _("Add new contact theme")
    icon_css_class = 'fa fa-fw fa-tags'
    dialog_class = 'modal-large'

    fields = field.Fields(IContactTheme).omit('__parent__', '__name__')
    edit_permission = MANAGE_TOOL_PERMISSION

    def create(self, data):
        return ContactTheme()

    def add(self, object):
        name = translate_string(object.name, spaces='-')
        IContactThemeManager(self.context)[name] = object

    def nextURL(self):
        return '#contact-themes.html'


@subscriber(IDataExtractedEvent, form_selector=ContactThemeAddForm)
def handle_contact_theme_add_form_data_extraction(event):
    """Check new theme for existing name"""
    context = event.form.context
    manager = IContactThemeManager(context)
    name = event.data.get('name')
    if translate_string(name, spaces='-') in manager:
        event.form.widgets.errors += (Invalid(_("Specified contact theme name is already used!")),)


@pagelet_config(name='properties.html', context=IContactTheme, layer=IPyAMSLayer,
                permission=MANAGE_TOOL_PERMISSION)
@ajax_config(name='properties.json', context=IContactTheme, layer=IPyAMSLayer)
class ContactThemeEditForm(AdminDialogEditForm):
    """Contact theme properties edit form"""

    prefix = 'contact_theme_properties.'

    legend = _("Contact theme properties")
    icon_css_class = 'fa fa-fw fa-tags'
    dialog_class = 'modal-large'

    fields = field.Fields(IContactTheme).omit('__parent__', '__name__')
    edit_permission = MANAGE_TOOL_PERMISSION

    def updateWidgets(self, prefix=None):
        super(ContactThemeEditForm, self).updateWidgets(prefix)
        if 'name' in self.widgets:
            self.widgets['name'].mode = DISPLAY_MODE

    def get_ajax_output(self, changes):
        output = super(ContactThemeEditForm, self).get_ajax_output(changes)
        if 'label' in changes.get(IContactTheme, ()):
            context = get_parent(self.context, IContactThemeManagerTarget)
            output.setdefault('events', []).append(
                get_json_table_row_refresh_event(context, self.request,
                                                 ContactThemeManagerView.table_class, self.context))
        return output
